﻿// Copyright (C) 2022 GlavSoft LLC.
// All rights reserved.
//
 
using System.Windows.Forms;
using GlavSoft.RemoteCoreSDK.Viewer.Input;

// ReSharper disable once CheckNamespace
namespace GlavSoft.RemoteCoreSDK.Viewer.Demo.Input
{
    /// <summary>
    /// The MouseController.
    /// It handles mouse events and then send them through <see cref="IRFBPointerEventsSender"/>
    /// </summary>
    class MouseController
    {
        #region - Private fields -

        /// <summary>
        /// bitmask for buttons
        /// </summary>
        private byte _buttonMask;

        /// <summary>
        /// Current mouse X position
        /// </summary>
        private int _x;

        /// <summary>
        /// Current mouse Y position
        /// </summary>
        private int _y;

        /// <summary>
        /// A point to interact with server side
        /// </summary>
        private readonly IRFBPointerEventsSender _sender;

        #endregion - Private fields -

        /// <summary>
        /// Initialize an entitity of <see cref="MouseController"/>
        /// </summary>
        /// <param name="sender"></param>
        internal MouseController(IRFBPointerEventsSender sender)
        {
            _buttonMask = 0;
            _sender = sender;
        }

        #region Mouse Handlers

        /// <summary>
        /// Handler for MouseDown event.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        internal async void MouseDownHandler(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Left)
                _buttonMask |= 0x1;
            if (e.Button == MouseButtons.Middle)
                _buttonMask |= 0x2;
            if (e.Button == MouseButtons.Right)
                _buttonMask |= 0x4;
            await _sender.SendPointerEventAsync(_buttonMask, e.Location.X, e.Location.Y).ConfigureAwait(false);
        }

        /// <summary>
        /// Handler for MouseUp event.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        internal async void MouseUpHandler(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Left)
                _buttonMask &= 0xFE;
            if (e.Button == MouseButtons.Middle)
                _buttonMask &= 0xFD; 
            if (e.Button == MouseButtons.Right)
                _buttonMask &= 0xFB;
            await _sender.SendPointerEventAsync(_buttonMask, e.Location.X, e.Location.Y).ConfigureAwait(false);
        }

        /// <summary>
        /// Handler for MouseWheel event.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        internal async void MouseWheelHandler(object sender, MouseEventArgs e)
        {
            if (e.Delta > 0)
            {
                _buttonMask |= 8;
                await _sender.SendPointerEventAsync(_buttonMask, _x, _y).ConfigureAwait(false);
                _buttonMask &= byte.MaxValue - 8;
                await _sender.SendPointerEventAsync(_buttonMask, _x, _y).ConfigureAwait(false);
            }
            else
                if (e.Delta < 0)
                {
                    _buttonMask |= 16;
                    await _sender.SendPointerEventAsync(_buttonMask, _x, _y).ConfigureAwait(false);
                    _buttonMask &= byte.MaxValue - 16;
                    await _sender.SendPointerEventAsync(_buttonMask, _x, _y).ConfigureAwait(false);
                }
                else
                {
                    await _sender.SendPointerEventAsync(_buttonMask, _x, _y).ConfigureAwait(false);
                }
        }

        /// <summary>
        /// Handler for MouseMove event.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        internal async void MouseMoveHandler(object sender, MouseEventArgs e)
        {
            if (e.X != _x || e.Y != _y)
            {
                await _sender.SendPointerEventAsync(_buttonMask, e.Location.X, e.Location.Y).ConfigureAwait(false);
                _x = e.X;
                _y = e.Y;
            }
        }

        #endregion

        internal void Clear()
        {
            _buttonMask = 0;
        }
    }
}
