﻿// Copyright (C) 2022 GlavSoft LLC.
// All rights reserved.
//


using System;
using System.Drawing;
using System.Drawing.Imaging;
using System.Windows.Forms;
using GlavSoft.RemoteCoreSDK.Constants;
using GlavSoft.RemoteCoreSDK.Viewer.Demo.Dialogs;
using GlavSoft.RemoteCoreSDK.Viewer.Demo.Input;
using GlavSoft.RemoteCoreSDK.Viewer.Drawing;
using GlavSoft.RemoteCoreSDK.Viewer.Exceptions;

//This is simple example of using .NetRemoteCoreSDK
namespace GlavSoft.RemoteCoreSDK.Viewer.Demo
{
    public partial class MainForm : Form
    {
        //The main class that allows you establish connection with 
        //remote host, request and receive updates of frame buffer,
        //send inputs events, etc.
        private IViewerCore _core;

        //Input controllers provide convenient way to send input's
        //events to the ViewerCore
        private MouseController _mc;
        private KeyboardController _kc;

        //Address and port of Remote host
        private readonly string _address = Helper.GetAddressFromSettings();
        private readonly ushort _port = Helper.GetPortFromSettings();

        //This bitmap will contains same pixels like a frame buffer.
        //It will be updated when the frame buffer changes.
        Bitmap _bitmap;
        private Graphics _graphics;

        //_bitmap will be used by different threads.
        //We create this object to use it like lock object for _bitmap
        private readonly object _bitmapLock = new object();

        public MainForm()
        {
            InitializeComponent();
        }

        private async void MainForm_Load(object sender, EventArgs e)
        {
            _bitmap = new Bitmap(1, 1);

            _core = ViewerCoreFactory.Create();
            _core.SetLogger(new SimpleLogger(LogLevel.Info));

            var eventsProvider = _core.EventsProvider;

            //Attaching of core events
            eventsProvider.FrameBufferSizeChanged += FrameBufferSizeChangedHandler;
            eventsProvider.FrameBufferUpdated += FrameBufferUpdatedHandler;
            eventsProvider.RequestPassword += RequestPasswordHandler;
            eventsProvider.Error += ErrorHandler;
            eventsProvider.AuthError += AuthErrorHandler;

            #region - Viewer settings (optional) -

            _core.AllowCopyRect(true);
            _core.SetJpegQuality(JpegQualityCode.JpegLevel6);
            _core.SetPreferredEncoding(EncodingTypes.Tight);
            _core.SetCompression(CompressionCode.Auto);

            #endregion
            
            //Starts via specified IP-address and port
            _core.IsSessionShared = true;
            await _core.StartAsync(_address, _port);

            //Initialization of inputs controllers
            _mc = new MouseController(_core);
            _kc = new KeyboardController(_core);

            AttachEvents();
        }

        /// <summary>
        /// Attaches input events to input controllers
        /// </summary>
        private void AttachEvents()
        {
            KeyDown += _kc.KeyDownHandler;
            KeyUp += _kc.KeyUpHandler;
            KeyPress += _kc.KeyPressHandler;

            pictureBox1.MouseMove += _mc.MouseMoveHandler;
            pictureBox1.MouseDown += _mc.MouseDownHandler;
            pictureBox1.MouseUp += _mc.MouseUpHandler;
        }

        /// <summary>
        /// Detaches input events
        /// </summary>
        private void DetachEvents()
        {
            KeyDown -= _kc.KeyDownHandler;
            KeyUp -= _kc.KeyUpHandler;
            KeyPress -= _kc.KeyPressHandler;

            pictureBox1.MouseMove -= _mc.MouseMoveHandler;
            pictureBox1.MouseDown -= _mc.MouseDownHandler;
            pictureBox1.MouseUp -= _mc.MouseUpHandler;
        }

        /// <summary>
        /// It calls when some error was occurred.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="args"></param>
        private void ErrorHandler(object sender, ErrorEventArgs args)
        {
            ShowErrorDialogAndClose(@"Error", args.Message);
        }

        /// <summary>
        /// It calls when some authentication error was occurred.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="args"></param>
        private void AuthErrorHandler(object sender, AuthErrorEventArgs args)
        {
            if (args.Type == AuthErrorType.AuthCanceled)
                BeginInvoke((MethodInvoker) Close);
            else
                ShowErrorDialogAndClose(@"Authentication error", args.Message);
        }


        private void FrameBufferUpdatedHandler(object sender, FrameBufferUpdatedEventArgs args)
        {
            var rect = args.UpdatedRectangle;

            args.FramebufferProvider.Reserve(fb =>
            {
                lock (_bitmapLock)
                {
                    var bmd = _bitmap.LockBits(new Rectangle(0, 0, _bitmap.Width, _bitmap.Height), ImageLockMode.WriteOnly, PixelFormat.Format32bppRgb);
                    try
                    {
                        bmd.Scan0.Synchronize(rect, fb);
                    }
                    finally
                    {
                        _bitmap.UnlockBits(bmd);
                    }
                }
            });

            BeginInvoke((MethodInvoker)(() =>
            {
                pictureBox1.Invalidate(rect);
            }));
        }

        /// <summary>
        /// It calls when size of frame buffer was changed.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="args"></param>
        private void FrameBufferSizeChangedHandler(object sender, FrameBufferSizeChangedEventArgs args)
        {
            lock (_bitmapLock)
            {
                //change size of _bitmap
                _bitmap = new Bitmap(args.Width, args.Height, args.CurrentPixelFormat);
                _graphics = Graphics.FromImage(_bitmap);
            }
        }

        /// <summary>
        /// It calls when password was requested from host
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="eventArgs"></param>
        private void RequestPasswordHandler(object sender, RequestPasswordEventArgs eventArgs)
        {
            string password = "";

            //you can pass the password through the command line as 3rd argument without any prefixes
            string[] args = Environment.GetCommandLineArgs();

            if (args.Length >= 4)
                password = args[3];
            else
            {
                DialogResult result = DialogResult.None;

                Invoke((MethodInvoker)(() =>
                {
                    var authDialog = new AuthDialog();
                    result = authDialog.ShowDialog();

                    password = authDialog.Password;
                }));

                if (result == DialogResult.Cancel)
                    throw new AuthenticationCanceledException();
            }
            eventArgs.Password = password;
        }

        private void ViewerMainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            DetachEvents();

            if (_kc != null)
                _kc.Dispose();

            if (_core != null)
                _core.Dispose();

            if (_bitmap != null)
                _bitmap.Dispose();
            
            if (_graphics != null)
                _graphics.Dispose();
        }

        private void pictureBox1_Paint(object sender, PaintEventArgs e)
        {
            lock (_bitmapLock)
            {
                pictureBox1.Size = new Size(_bitmap.Width, _bitmap.Height);
                e.Graphics.DrawImage(_bitmap, e.ClipRectangle.X, e.ClipRectangle.Y , e.ClipRectangle, GraphicsUnit.Pixel);
            }
        }

        /// <summary>
        /// We use it when we need to show dialog from non-UI thread and close then (e.g. from core events handlers) 
        /// </summary>
        /// <param name="tittle"></param>
        /// <param name="message"></param>
        private void ShowErrorDialogAndClose(string tittle, string message)
        {
            BeginInvoke((MethodInvoker)(() =>
            {
                MessageBox.Show(message, tittle, MessageBoxButtons.OK, MessageBoxIcon.Error);
                Close();
            }));
        }
    }
}
