﻿// Copyright (C) 2022 GlavSoft LLC.
// All rights reserved.
//
// DO NOT USE OR REDISTRIBUTE THE CODE, UNLESS YOU HAVE OBTAINED A LICENSE.
//

using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Runtime.InteropServices;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using GlavSoft.RemoteCoreSDK.Viewer.Data;
using GlavSoft.RemoteCoreSDK.Extensions.Loggers.NLog;
using GlavSoft.RemoteCoreSDK.Viewer.Demo.Configuration;
using GlavSoft.RemoteCoreSDK.Viewer.WPF.Configuration;
using GlavSoft.RemoteCoreSDK.Viewer.WPF.Demo.Configuration;
using WPF.Demo;
using PixelFormat = System.Drawing.Imaging.PixelFormat;

// ReSharper disable once CheckNamespace
namespace GlavSoft.RemoteCoreSDK.Viewer.WPF.Demo
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow
    {
        private const bool PredefinedDiagnostic = false;

        private readonly Settings _settingsWindow;
        private ViewerConfiguration _configuration;
        private bool _hasConnected;
        private NLogLogger _logger;

        public MainWindow()
        {
            InitializeComponent();

            FullScreenCommand = new RelayCommand(OnFullScreen);
            StartCommand = new RelayCommand(OnStart);
            IsFullScreen = false;
            _settingsWindow = new Settings(DesktopControl, _logger);
            DesktopControl.AuthenticationError += OnAuthError;
            DesktopControl.Error += OnError;
            DesktopControl.Connect += OnConnected;
            DesktopControl.Disconnect += (_, args) => _hasConnected = false;
        }

        public static readonly DependencyProperty FullScreenCommandProperty = DependencyProperty.Register(
            "FullScreenCommand", typeof(ICommand), typeof(MainWindow), new PropertyMetadata(default(ICommand)));

        public static readonly DependencyProperty StartProperty = DependencyProperty.Register(
            "PasswordCommand", typeof(ICommand), typeof(PasswordBox), new PropertyMetadata(default(ICommand)));

        public static readonly DependencyProperty IsFullScreenProperty = DependencyProperty.Register(
            "IsFullScreen", typeof(bool), typeof(MainWindow), new PropertyMetadata(default(bool)));

        public static readonly DependencyProperty ShowSettingMenuProperty = DependencyProperty.Register(
            "ShowSettingMenu", typeof(bool), typeof(MainWindow));

        public static readonly DependencyProperty IsAuthFailedProperty = DependencyProperty.Register(
            "IsAuthFailed", typeof(bool), typeof(MainWindow));

        public static readonly DependencyProperty DiagnosticModeProperty = DependencyProperty.Register(
            nameof(DiagnosticMode), typeof(bool), typeof(MainWindow), new PropertyMetadata(PredefinedDiagnostic));

        public bool IsFullScreen
        {
            get => (bool)GetValue(IsFullScreenProperty);
            set => SetValue(IsFullScreenProperty, value);
        }

        public bool IsAuthFailed
        {
            get => (bool)GetValue(IsAuthFailedProperty);
            set => SetValue(IsAuthFailedProperty, value);
        }

        public bool ShowSettingMenu
        {
            get => (bool)GetValue(ShowSettingMenuProperty);
            set => SetValue(ShowSettingMenuProperty, value);
        }

        public ICommand FullScreenCommand
        {
            get => (ICommand)GetValue(FullScreenCommandProperty);
            set
            {
                SetValue(FullScreenCommandProperty, value);
                IsFullScreen = !IsFullScreen;
            }
        }

        public ICommand StartCommand
        {
            get => (ICommand)GetValue(StartProperty);
            set => SetValue(StartProperty, value);
        }

        public bool DiagnosticMode
        {
            get => (bool)GetValue(DiagnosticModeProperty);
            set => SetValue(DiagnosticModeProperty, value);
        }

        protected override void OnClosed(EventArgs e)
        {
            DesktopControl.Dispose();
            base.OnClosed(e);
            Application.Current.Shutdown();
        }

        protected override void OnInitialized(EventArgs e)
        {
            base.OnInitialized(e);
#if DEV
            DesktopControl.SetUpDevSettings();
#endif
            _configuration = ConfigurationProvider.ReadConfiguration();
            DesktopControl.ApplyConfiguration(_configuration);
            DesktopControl.Captured += DesktopControlOnCaptured;

#pragma warning disable CS0162 // Unreachable code detected
            // ReSharper disable All
            if (PredefinedDiagnostic)
            {
                LoggingConfiguration.UseDefault();
                _logger = new NLogLogger(LogLevel.Trace);
                DesktopControl.SetLogger(_logger);
            }
            // ReSharper restore All
#pragma warning restore CS0162 // Unreachable code detected
        }

        protected override void OnClosing(CancelEventArgs e)
        {
            ConfigurationProvider.SaveConfiguration(DesktopControl.GetConfiguration());
        }

        private void DesktopControlOnCaptured(object o, FrameBufferCapturedEventArgs args)
        {
            try
            {
                var bitmap = args.RemoteFrameBuffer.Buffer;
                ConvertBytestoJpegBytes(bitmap, args.RemoteFrameBuffer.Width, args.RemoteFrameBuffer.Height);
            }
            catch (Exception e)
            {
                Debug.Write(e);
            }
        }

        private void ConvertBytestoJpegBytes(byte[] pixels24Bpp, int w, int h)
        {
            var gch = GCHandle.Alloc(pixels24Bpp, GCHandleType.Pinned);
            var stride = 4 * w;
            var bmp = new Bitmap(w, h, stride, PixelFormat.Format32bppArgb, gch.AddrOfPinnedObject());
            const string directory = "screenshots";

            if (!Directory.Exists(directory))
            {
                Directory.CreateDirectory(directory);
            }

            var path = Path.Combine("screenshots", $"{DateTime.Now:ddMMyy-hhmmss}.png");
            bmp.Save(path, ImageFormat.Png);
            gch.Free();
        }

        private void OnFullScreen(object obj)
        {
            if (!IsFullScreen)
            {
                IsFullScreen = true;
                DesktopControl.RegisterLowLevelHooks();
                WindowState = WindowState.Normal;
                WindowStyle = WindowStyle.None;
                WindowState = WindowState.Maximized;
            }
            else
            {
                IsFullScreen = false;
                DesktopControl.UnregisterLowLevelHooks();
                WindowStyle = WindowStyle.SingleBorderWindow;
                WindowState = WindowState.Normal;
            }

            DesktopControl.Focus();
        }

        private void OnStart(object sender)
        {
            if (!(sender is PasswordBox passwordBox))
                return;

            DesktopControl.Password = passwordBox.Password;
            DesktopControl.StartCommand.Execute(this);
        }

        private void OnClickSettings(object sender, RoutedEventArgs e)
        {
            _settingsWindow.Owner = this;
            _settingsWindow.WindowStartupLocation = WindowStartupLocation.CenterOwner;
            _settingsWindow.Show();
            _settingsWindow.Focus();
        }

        private void OnPopupClick(object sender, RoutedEventArgs e)
        {
            if (!(sender is Button popupButton))
                return;
            Popup.PlacementTarget = popupButton.Name == "FullScreenSetting" ? FullScreenSetting : popupButton;
            ShowSettingMenu = !ShowSettingMenu;
        }

        private void OnPopupPreviewMouseUp(object sender, MouseButtonEventArgs e)
        {
            ShowSettingMenu = false;
        }

        private void OnConnected(object sender, EventArgs e)
        {
            _hasConnected = true;
            IsAuthFailed = false;
        }

        private void OnError(object sender, ErrorEventArgs e)
        {
            if (!_hasConnected && DesktopControl.AutoReconnect != AutoReconnectMode.Always)
            {
                DesktopControl.StopCommand.Execute(this);
            }
        }

        private void OnAuthError(object sender, ErrorEventArgs e)
        {
            DesktopControl.StopCommand.Execute(this);
            IsAuthFailed = true;
        }
    }
}