﻿// Copyright (C) 2022 GlavSoft LLC.
// All rights reserved.
//
// DO NOT USE OR REDISTRIBUTE THE CODE, UNLESS YOU HAVE OBTAINED A LICENSE.
//

using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using GlavSoft.RemoteCoreSDK.Viewer;
using GlavSoft.RemoteCoreSDK.Viewer.Data;
using GlavSoft.Viewer.WinForms.Configuration;
using GlavSoft.Viewer.WinForms.Demo.Configuration;
using WinForms.Demo;

// ReSharper disable once CheckNamespace
namespace GlavSoft.Viewer.WinForms.Demo
{
    internal struct LayoutStyle
    {
        public AnchorStyles Anchor;
        public DockStyle Dock;
        public Size Size;
        public Size MaxSize;
    }

    enum HooksState
    {
        Disabled = 0,
        Registered = 1,
        Auto = 2
    }

    public partial class MainForm : Form
    {
        private bool _isFullScreen;
        private LayoutStyle _remoteDesktopLayout;
        private object _hooksComboBoxState;
        private readonly Settings _settingsWindow;
        private bool _isError;

        public MainForm()
        {
            InitializeComponent();
            _settingsWindow = new Settings(remoteDesktop);
            regHooksComboBox.DataSource = Enum.GetValues(typeof(HooksState)).Cast<HooksState>().ToList();
            var config = ConfigurationProvider.ReadConfiguration();
            remoteDesktop.ApplyConfiguration(config);

            remoteDesktop.Captured += DesktopControlOnCaptured;
            remoteDesktop.Connect += (sender, args) =>
            {
                connectionStatusLabel.Text = @"Connection status: Connected";
                GetElementsToConnectedState();
            };
            remoteDesktop.Disconnect +=
                (sender, args) => connectionStatusLabel.Text = @"Connection status: Disconnected";

            remoteDesktop.Error += (sender, args) =>
            {
                _isError = true;

                MessageBox.Show(args.Message, @"Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                connectionStatusLabel.Text = @"Connection status: Disconnected";
                GetElementsToReadyState();
                switch (remoteDesktop.AutoReconnect)
                {
                    case AutoReconnectMode.Always:
                    case AutoReconnectMode.OnlyAfterSuccessfulConnection when remoteDesktop.IsConnected:
                        return;
                }

                remoteDesktop.Stop();
                stopButton.Enabled = false;
                complexPanel.Show();
                label2.Hide();
                GetElementsToReadyState();
            };

#if DEV
            remoteDesktop.SetUpDevSettings();
#endif
            remoteDesktop.ClipboardEnabled = true;
        }

        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);
            isSharedCheckBox.Checked = remoteDesktop.IsSessionShared;
            isAutoScalingCheckBox.Checked = remoteDesktop.IsAutoScale;
            ignoreInputCheckBox.Checked = remoteDesktop.IgnoreInput;
            regHooksComboBox.SelectedItem =
                remoteDesktop.AutoRegisterLowLevelHooks ? HooksState.Auto : HooksState.Disabled;
            scaleUpDownBar.Value = remoteDesktop.Scale;
            portTextBox.Text = remoteDesktop.Port.ToString();
            hostTextBox.Text = remoteDesktop.RemoteHostAddress;
            scaleUpDownBar.Value = remoteDesktop.Scale;
            pauseCheckBox.Checked = remoteDesktop.PauseUpdates;
            label2.Hide();
            GetElementsToReadyState();
            settingFullScreenButton.Hide();
            leaveFullScreenButton.Enabled = false;
            sendCadButton.Enabled = false;
            captureButton.Enabled = false;
            pauseCheckBox.Enabled = false;

            remoteDesktop.AuthenticationError += OnAuthError;
        }

        private void OnAuthError(object sender, RemoteCoreSDK.Viewer.ErrorEventArgs e)
        {
            complexPanel.Show();
            label2.Show();
            GetElementsToReadyState();
        }

        private void OnError(object sender, RemoteCoreSDK.Viewer.ErrorEventArgs e)
        {
            switch (remoteDesktop.AutoReconnect)
            {
                case AutoReconnectMode.Always:
                case AutoReconnectMode.OnlyAfterSuccessfulConnection when remoteDesktop.IsConnected:
                    return;
            }

            complexPanel.Show();
            label2.Hide();
            GetElementsToReadyState();
        }

        private void StartClick(object sender, EventArgs e)
        {
            remoteDesktop.RemoteHostAddress = hostTextBox.Text;

            var success = TryGetUnsignedValue(portTextBox, out var result);
            if (!success)
            {
                Debug.WriteLine("Try to enter invalid port number. Start command is cancelled");
                StopClick(this, null);
                return;
            }

            if (passwordBox.Text != null)
            {
                remoteDesktop.PasswordRequested += (o, args) => args.Password = passwordBox.Text;
            }

            remoteDesktop.Port = result;
            remoteDesktop.Scale = scaleUpDownBar.Value;
            remoteDesktop.Start();
            complexPanel.Hide();
            stopButton.Enabled = true;
        }

        private void GetElementsToConnectedState()
        {
            stopButton.Enabled = true;
            sendCadButton.Enabled = true;
            captureButton.Enabled = true;
            pauseCheckBox.Enabled = true;
            hostTextBox.Enabled = false;
            portTextBox.Enabled = false;
            startButton.Enabled = false;
            passwordBox.Enabled = false;
        }

        private void StopClick(object sender, EventArgs e)
        {
            remoteDesktop.Stop();
            stopButton.Enabled = false;
            complexPanel.Show();
            label2.Hide();
            GetElementsToReadyState();
        }

        private void GetElementsToReadyState()
        {
            if (!(remoteDesktop.AutoReconnect != AutoReconnectMode.Disabled && _isError))
            {
                stopButton.Enabled = false;
            }
            sendCadButton.Enabled = false;
            captureButton.Enabled = false;
            pauseCheckBox.Enabled = false;
            hostTextBox.Enabled = true;
            portTextBox.Enabled = true;
            startButton.Enabled = true;
            passwordBox.Enabled = true;
        }

        private void CaptureClick(object sender, EventArgs e)
        {
            remoteDesktop.CaptureScreen();
        }

        private void SendCadClick(object sender, EventArgs e)
        {
            remoteDesktop.SendCad();
        }

        private void ScaleUpDownScroll(object sender, EventArgs e)
        {
            remoteDesktop.Scale = Convert.ToInt32(scaleUpDownBar.Value);
        }

        private void PauseCheckedChanged(object sender, EventArgs e)
        {
            remoteDesktop.PauseUpdates = pauseCheckBox.Checked;
        }

        private void FullScreenButtonClick(object sender, EventArgs e)
        {
            if (!_isFullScreen)
            {
                mainPanel.Hide();
                settingFullScreenButton.Show();
                leaveFullScreenButton.Enabled = true;
                _hooksComboBoxState = regHooksComboBox.SelectedItem;
                regHooksComboBox.SelectedItem = HooksState.Registered;
                regHooksComboBox.Enabled = false;

                _isFullScreen = true;

                WindowState = FormWindowState.Normal;
                FormBorderStyle = FormBorderStyle.None;
                WindowState = FormWindowState.Maximized;

                _remoteDesktopLayout.Anchor = remoteDesktop.Anchor;
                _remoteDesktopLayout.Dock = remoteDesktop.Dock;
                _remoteDesktopLayout.MaxSize = remoteDesktop.MaximumSize;
                _remoteDesktopLayout.Size = remoteDesktop.Size;

                remoteDesktop.MaximumSize =
                    new Size(_remoteDesktopLayout.MaxSize.Width, _remoteDesktopLayout.MaxSize.Height);
                remoteDesktop.Dock = DockStyle.Fill;
            }
            else
            {
                mainPanel.Show();
                settingFullScreenButton.Hide();
                leaveFullScreenButton.Enabled = false;
                regHooksComboBox.Enabled = true;
                regHooksComboBox.SelectedItem = _hooksComboBoxState;

                _isFullScreen = false;

                remoteDesktop.Dock = _remoteDesktopLayout.Dock;
                remoteDesktop.MaximumSize = _remoteDesktopLayout.MaxSize;
                remoteDesktop.Size = _remoteDesktopLayout.Size;
                remoteDesktop.Anchor = _remoteDesktopLayout.Anchor;

                FormBorderStyle = FormBorderStyle.Sizable;
                WindowState = FormWindowState.Normal;
            }

            remoteDesktop.Focus();
        }

        protected override void OnClosing(CancelEventArgs e)
        {
            ConfigurationProvider.SaveConfiguration(remoteDesktop.GetConfiguration());
        }

        public bool TryGetUnsignedValue(TextBox textBox, out int result)
        {
            var success = int.TryParse(textBox.Text, out var parseResult);

            if (success && parseResult >= 0)
            {
                result = parseResult;
                return true;
            }

            result = 0;
            return false;
        }

        private void DesktopControlOnCaptured(object o, FrameBufferCapturedEventArgs args)
        {
            try
            {
                var bitmap = args.RemoteFrameBuffer.Buffer;
                ConvertBytestoJpegBytes(bitmap, args.RemoteFrameBuffer.Width, args.RemoteFrameBuffer.Height);
            }
            catch (Exception e)
            {
                Debug.Write(e);
            }
        }

        private void ConvertBytestoJpegBytes(byte[] pixels24Bpp, int w, int h)
        {
            var gch = GCHandle.Alloc(pixels24Bpp, GCHandleType.Pinned);
            try
            {
                var stride = 4 * w;
                var bmp = new Bitmap(w, h, stride, PixelFormat.Format32bppArgb, gch.AddrOfPinnedObject());
                var directory = "screenshots";

                if (!Directory.Exists(directory))
                {
                    Directory.CreateDirectory(directory);
                }

                var path = Path.Combine("screenshots", $"{DateTime.Now:ddMMyy-hhmmss}.png");
                bmp.Save(path, ImageFormat.Png);
            }
            finally
            {
                gch.Free();
            }
        }

        protected override bool ProcessCmdKey(ref Message msg, Keys keyData)
        {
            if (keyData == (Keys.Control | Keys.Alt | Keys.Shift | Keys.F))
            {
                FullScreenButtonClick(this, null);
                return true;
            }

            return base.ProcessCmdKey(ref msg, keyData);
        }

        private void RemoteDesktop_CutText(object sender, CutTextEventArgs e)
        {
            hostTextBox.Text = e.Text;
        }

        private void RegHooksComboBoxSelectedIndexChanged(object sender, EventArgs e)
        {
            switch (regHooksComboBox.SelectedItem)
            {
                case HooksState.Registered:
                    remoteDesktop.RegisterLowLevelHooks();
                    remoteDesktop.AutoRegisterLowLevelHooks = false;
                    break;
                case HooksState.Disabled:
                    remoteDesktop.UnRegisterLowLevelHooks();
                    remoteDesktop.AutoRegisterLowLevelHooks = false;
                    break;
                case HooksState.Auto:
                    remoteDesktop.AutoRegisterLowLevelHooks = true;
                    break;
            }
        }

        private void IgnoreInputCheckedChanged(object sender, EventArgs e)
        {
            remoteDesktop.IgnoreInput = ignoreInputCheckBox.Checked;
        }

        private void IsSharedCheckedChanged(object sender, EventArgs e)
        {
            remoteDesktop.IsSessionShared = isSharedCheckBox.Checked;
        }

        private void AdvanceSettingClick(object sender, EventArgs e)
        {
            _settingsWindow.Owner = this;
            _settingsWindow.Show();
        }

        private void IsAutoScalingCheckedChanged(object sender, EventArgs e)
        {
            remoteDesktop.IsAutoScale = isAutoScalingCheckBox.Checked;
        }

        private void ContextMenuStripClick(object sender, EventArgs e)
        {
            if (!(sender is Button button))
                return;

            switch (button.Name)
            {
                case "settingFullScreenButton":
                    commandContextMenu.Show(settingFullScreenButton, new Point(0, settingFullScreenButton.Height));
                    break;
                case "settingButton":
                    commandContextMenu.Show(settingButton, new Point(0, settingButton.Height));
                    break;
            }
        }
    }
}