﻿// Copyright (C) 2022 GlavSoft LLC.
// All rights reserved.
//
// DO NOT USE OR REDISTRIBUTE THE CODE, UNLESS YOU HAVE OBTAINED A LICENSE.
//

using System;
using System.Diagnostics;
using System.Drawing;
using System.Linq;
using System.Windows.Forms;
using GlavSoft.RemoteCoreSDK.Extensions.Loggers.NLog;
using GlavSoft.RemoteCoreSDK.Viewer.Data;
using GlavSoft.RemoteCoreSDK.Viewer.Demo.Configuration;
using GlavSoft.Viewer.WinForms;
using GlavSoft.Viewer.WinForms.Demo;
using LogLevel = GlavSoft.RemoteCoreSDK.LogLevel;

namespace WinForms.Demo
{
    public partial class Settings : Form
    {
        private readonly RemoteDesktop _remoteDesktop;
        private bool _enterKeyDown;
        private bool _isShownAttentionMsg;

        public Settings(RemoteDesktop remoteDesktop)
        {
            InitializeComponent();
            _remoteDesktop = remoteDesktop;
            preferredEncodingComboBox.DataSource =
                Enum.GetValues(typeof(ViewerEncodings)).Cast<ViewerEncodings>().ToList();
            pixelFormatComboBox.DataSource = Enum.GetValues(typeof(BppType)).Cast<BppType>().ToList();
            _remoteDesktop.Connect += (o, args) => DiagnosticModeCheckBox.Enabled = false;
        }

        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);
            preferredEncodingComboBox.SelectedItem = _remoteDesktop.PreferredEncoding;
            disabledAutoReconnectRadioButton.Checked = true;
            pixelFormatComboBox.SelectedItem = _remoteDesktop.PixelFormat;
            allowCopyRectCheckBox.Checked = _remoteDesktop.AllowCopyRect;
            allowJpegCheckBox.Checked = _remoteDesktop.AllowJPEG;
            setComressionCheckBox.Checked = _remoteDesktop.CustomCompressionLevelEnabled;
            сustomCompressionLevelTrackBar.Value = (int) _remoteDesktop.CustomCompressionLevel;
            jpegQualityLevelTrackBar.Value = (int) _remoteDesktop.JpegQualityLevel;
            updateRequestsDelayTextBox.Text = Convert.ToString(_remoteDesktop.UpdateRequestsDelay);
            enableTightExtBox.Checked = _remoteDesktop.TightSecurityEnabled;
            flushingIntervalTextBox.Text = _remoteDesktop.UpdatesFlushingTimeout.ToString();
            updatesCollectingCheckBox.Checked = _remoteDesktop.UseUpdatesCollecting;

            switch (_remoteDesktop.CursorTrackingMode)
            {
                case CursorTrackingMode.TrackLocally:
                    сursorTrackLocallyRadioButton.Checked = true;
                    break;
                case CursorTrackingMode.TrackRemote:
                    cursorTrackRemoteRadioButton.Checked = true;
                    break;
                case CursorTrackingMode.DontShow:
                    cursorHideRadioButton.Checked = true;
                    break;
            }

            сustomCompressionLevelTrackBar.Enabled = setComressionCheckBox.Checked;
            jpegQualityLevelTrackBar.Enabled = allowJpegCheckBox.Checked;
        }

        private void PreferredEncodingSelectedIndexChanged(object sender, EventArgs e)
        {
            _remoteDesktop.PreferredEncoding = (ViewerEncodings) preferredEncodingComboBox.SelectedItem;
        }

        private void PixelFormatSelectedIndexChanged(object sender, EventArgs e)
        {
            _remoteDesktop.PixelFormat = (BppType) pixelFormatComboBox.SelectedItem;
        }

        private void AllowCopyRectCheckedChanged(object sender, EventArgs e)
        {
            _remoteDesktop.AllowCopyRect = allowCopyRectCheckBox.Checked;
        }

        private void SetCompressionCheckedChanged(object sender, EventArgs e)
        {
            _remoteDesktop.CustomCompressionLevelEnabled = setComressionCheckBox.Checked;
            сustomCompressionLevelTrackBar.Enabled = setComressionCheckBox.Checked;
        }

        private void CustomCompressionLevelUpDownScroll(object sender, EventArgs e)
        {
            _remoteDesktop.CustomCompressionLevel = (CompressionLevel) сustomCompressionLevelTrackBar.Value;
        }

        private void JpegQualityLevelUpDownScroll(object sender, EventArgs e)
        {
            _remoteDesktop.JpegQualityLevel = (JpegQualityLevel) jpegQualityLevelTrackBar.Value;
        }

        private void CursorTrackLocallyCheckedChanged(object sender, EventArgs e)
        {
            _remoteDesktop.CursorTrackingMode = CursorTrackingMode.TrackLocally;
        }

        private void CursorTrackRemoteCheckedChanged(object sender, EventArgs e)
        {
            _remoteDesktop.CursorTrackingMode = CursorTrackingMode.TrackRemote;
        }

        private void CursorHideCheckedChanged(object sender, EventArgs e)
        {
            _remoteDesktop.CursorTrackingMode = CursorTrackingMode.DontShow;
        }

        private void EnableTightExtCheckedChanged(object sender, EventArgs e)
        {
            _remoteDesktop.TightSecurityEnabled = enableTightExtBox.Checked;
        }

        private void AllowJpegCheckedChanged(object sender, EventArgs e)
        {
            _remoteDesktop.AllowJPEG = allowJpegCheckBox.Checked;
            jpegQualityLevelTrackBar.Enabled = allowJpegCheckBox.Checked;
        }

        private void ForceFullUpdateCheckedChanged(object sender, EventArgs e)
        {
            _remoteDesktop.ForceFullUpdateRequests = forceFullUpdateCheckBox.Checked;
        }

        private void UseCollectingCheckedChanged(object sender, EventArgs e)
        {
            _remoteDesktop.UseUpdatesCollecting = updatesCollectingCheckBox.Checked;
        }

        private void UpdateRequestsDelayTextBoxKeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyData != Keys.Enter)
                return;

            _enterKeyDown = true;
            TryEnterDelayValue();
            ActiveControl = null;
        }

        private void UpdateRequestsDelayTextBoxLeave(object sender, EventArgs e)
        {
            if (_enterKeyDown)
            {
                _enterKeyDown = false;
                return;
            }

            TryEnterDelayValue();
        }

        private void UpdateFlushingTimeoutTextBoxKeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyData != Keys.Enter)
                return;

            _enterKeyDown = true;
            TryEnterFlushingTimeout();
            ActiveControl = null;
        }

        private void FlushingIntervalTextBoxLeave(object sender, EventArgs e)
        {
            if (_enterKeyDown)
            {
                _enterKeyDown = false;
                return;
            }

            TryEnterFlushingTimeout();
        }

        private void TryEnterFlushingTimeout()
        {
            if (!(Owner is MainForm main))
                return;

            var success = main.TryGetUnsignedValue(flushingIntervalTextBox, out var result);

            if (success)
            {
                _remoteDesktop.UpdatesFlushingTimeout = result;
            }
            else
            {
                Debug.WriteLine("Try to enter invalid flushing interval");
                updateRequestsDelayTextBox.Text = Convert.ToString(_remoteDesktop.UpdatesFlushingTimeout);
            }
        }

        private void TryEnterDelayValue()
        {
            if (!(Owner is MainForm main))
                return;

            var success = main.TryGetUnsignedValue(updateRequestsDelayTextBox, out var result);

            if (success)
            {
                _remoteDesktop.UpdateRequestsDelay = result;
            }
            else
            {
                Debug.WriteLine("Try to enter invalid update timeout");
                updateRequestsDelayTextBox.Text = Convert.ToString(_remoteDesktop.UpdateRequestsDelay);
            }
        }

        private void CloseClick(object sender, EventArgs e)
        {
            Hide();
            Owner?.Focus();
        }

        private void DiagnosticModeCheckedChanged(object sender, EventArgs e)
        {
            if (DiagnosticModeCheckBox.Checked)
            {
                LoggingConfiguration.UseDefault();
                _remoteDesktop.UseNLog(LogLevel.Trace);

                if (!_isShownAttentionMsg)
                {
                    MessageBox.Show(
                        this,
                        @"The Diagnostic mode can affect the image quality and responsiveness",
                        @"Attention",
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Warning);
                    _isShownAttentionMsg = true;
                }

                DiagnosticModeCheckBox.ForeColor = Color.Red;
            }
            else
            {
                _remoteDesktop.SetLogger(null);
                DiagnosticModeCheckBox.ForeColor = DefaultForeColor;
            }
        }

        private void AfterSuccessfulConnectionAutoReconnectCheckedChanged(object sender, EventArgs e)
        {
            _remoteDesktop.AutoReconnect = AutoReconnectMode.OnlyAfterSuccessfulConnection;
        }

        private void AlwaysAutoReconnectCheckedChanged(object sender, EventArgs e)
        {
            _remoteDesktop.AutoReconnect = AutoReconnectMode.Always;
        }

        private void DisabledAutoReconnectCheckedChanged(object sender, EventArgs e)
        {
            _remoteDesktop.AutoReconnect = AutoReconnectMode.Disabled;
        }
    }
}